@@ -1,16 +1,20 @@
 //! Stuff about the game map.
 
+mod render;
 mod terrain;
 
-use self::terrain::{generation::Config, Terrain, Terrains};
+use self::{
+    render::Lines,
+    terrain::{generation::Config, Terrain, Terrains},
+};
 use bevy::{
     core_pipeline::ClearColor,
     math::Vec3,
     prelude::{
-        App, AssetServer, Bundle, Color, Commands, Component, Entity, EventReader, EventWriter,
-        Handle, Image, OrthographicCameraBundle, Plugin, Res, ResMut, Transform,
+        App, AssetServer, Assets, Bundle, Color, Commands, Component, Entity, EventReader,
+        EventWriter, Handle, Image, Mesh, Plugin, Res, ResMut, Transform,
     },
-    sprite::SpriteBundle,
+    sprite::{ColorMaterial, ColorMesh2dBundle, Mesh2dHandle, SpriteBundle},
 };
 use hex2d::{Coordinate, Spacing};
 use std::collections::HashMap;
@@ -22,7 +26,7 @@ struct Location {
 }
 
 impl Location {
-    const SIZE: f32 = 4.0;
+    const SIZE: f32 = 1.0;
     const SPACING: Spacing = Spacing::PointyTop(Self::SIZE / 2.0);
 
     fn to_vec3(coord: Coordinate) -> Vec3 {
@@ -83,6 +87,7 @@ impl TileBundle {
 #[derive(Default)]
 struct Map {
     tiles: HashMap<Coordinate, Entity>,
+    frame: Option<Entity>,
 }
 
 pub(super) struct MapPlugin;
@@ -93,7 +98,6 @@ impl Plugin for MapPlugin {
         app.insert_resource(Terrains::default());
         app.insert_resource(Map::default());
         app.insert_resource(ClearColor(Color::rgb(0.94, 0.97, 1.0)));
-        app.add_startup_system(create_camera);
         app.add_startup_system(load_tile_texture);
         app.add_startup_system(generate_initial_map);
         app.add_system(generate_map);
@@ -104,10 +108,6 @@ impl Plugin for MapPlugin {
     }
 }
 
-fn create_camera(mut commands: Commands) {
-    commands.spawn_bundle(OrthographicCameraBundle::new_2d());
-}
-
 struct TileTexture(Handle<Image>);
 
 impl TileTexture {
@@ -130,13 +130,42 @@ fn generate_map(
     mut gen_event: EventReader<MapGenEvent>,
     terrains: Res<Terrains>,
     texture: Res<TileTexture>,
+    mut meshes: ResMut<Assets<Mesh>>,
+    mut colors: ResMut<Assets<ColorMaterial>>,
 ) {
     if gen_event.iter().next().is_some() {
         for (_, tile) in map.tiles.iter() {
             commands.entity(*tile).despawn();
         }
         map.tiles.clear();
+        if let Some(entity) = map.frame.take() {
+            commands.entity(entity).despawn();
+        }
         let terrain_map = Config::default().create(&terrains).generate();
+        let mesh = {
+            let mut lines = Lines::new(Location::SPACING);
+            for coord in terrain_map.keys() {
+                lines.add(*coord);
+            }
+            lines.into_mesh()
+        };
+        map.frame = Some(
+            commands
+                .spawn()
+                .insert_bundle(ColorMesh2dBundle {
+                    mesh: Mesh2dHandle(meshes.add(mesh)),
+                    material: colors.add(ColorMaterial {
+                        color: Color::rgb(0.0, 0.0, 0.0),
+                        ..Default::default()
+                    }),
+                    transform: Transform {
+                        translation: Vec3::new(0.0, 0.0, 1.0),
+                        ..Default::default()
+                    },
+                    ..Default::default()
+                })
+                .id(),
+        );
         for (coord, terrain) in terrain_map.into_iter() {
             let tile = commands
                 .spawn_bundle(TileBundle::new(coord, terrain, &terrains, &texture))
